<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) 2019 Amasty (https://www.amasty.com)
 * @package Amasty_Coupons
 *
 * FILOBLU - LOGIC CHANGED
 * @author - JORDY DE ROSA - jordy.derosa@filoblu.com
 */


namespace Amasty\Coupons\Observer;

use Magento\Framework\Event\ObserverInterface;
use Amasty\Coupons\Model\DiscountCollector;
use Magento\Framework\Event\Observer;
use Magento\SalesRule\Model\Rule;
use Magento\SalesRule\Model\Coupon;
use Amasty\Coupons\Helper\Data;


/**
 * Class CouponObserver
 */
class CouponObserver implements ObserverInterface
{
    /**
     * @var DiscountCollector
     */
    protected $discountCollector;
    protected $coupon;
    protected $saleRule;
    protected $itemIndex;
    protected $couponArray;
    protected $ruleIdArray;
    protected $helperData;


    /**
     * CouponObserver constructor.
     * @param DiscountCollector $discountCollector
     * @param Coupon $coupon
     * @param Rule $saleRule
     * @param Data $data
     */
    public function __construct(
        DiscountCollector $discountCollector,
        Coupon $coupon,
        Rule $saleRule,
        Data $data
    ) {
        $this->discountCollector = $discountCollector;
        $this->coupon = $coupon;
        $this->saleRule = $saleRule;
        $this->helperData = $data;
        $this->itemIndex = 0;
        $this->couponArray = [];
        $this->ruleIdArray = [];

    }




    /**
     * event salesrule_validator_process
     * @param Observer $observer
     */
    public function execute(Observer $observer)
    {
        if(!is_null($observer->getData('rule')->getCode()))
        {
            //reset amount to override discount
            $amount = 0;
            $observer->getResult()->setAmount($amount);

            //get quote items
            $quoteItems = $observer->getQuote()->getItems();
            $quoteCoupons = explode(",",$observer->getQuote()->getCouponCode());

            //get current rule data
            $ruleId = $this->coupon->loadByCode($observer->getData('rule')->getCode())->getRuleId();
            $rule = $this->saleRule->load($ruleId);
            $ruleSimpleAction= $rule->getSimpleAction() ?? '';
            $ruleDiscount = $rule->getDiscountAmount() ?? 0;

            if(!in_array($observer->getData('rule')->getCode(),$this->couponArray))
            {
                $this->couponArray [] = $observer->getData('rule')->getCode();
                if(!in_array($ruleId,$this->ruleIdArray))
                {
                    $this->ruleIdArray[] = $ruleId;
                }
            }
            else
            {
                $this->itemIndex++;
                if(!in_array($observer->getData('rule')->getCode(),$this->couponArray))
                {
                    $this->couponArray [] = $observer->getData('rule')->getCode();
                }
                else
                {
                    $this->itemIndex++;
                    $this->couponArray = [];
                }
            }

            //count how many same rule coupons are applied to the quote beacuse same rule coupons doesn't trigger observer
            $appliedCodes = explode(",",$observer->getData('quote')->getCouponCode());


            $ruleDescription = $this->helperData->getRuleLabelById($ruleId);
            $sameRuleCounter = $this->helperData->countSameRuleCoupons($appliedCodes,$ruleId)['count'];
            $ruleUsedCodes = $this->helperData->countSameRuleCoupons($appliedCodes,$ruleId)['coupons'];

            if($ruleDescription == "")
            {
                $ruleDescription = $ruleUsedCodes;
            }

            $item = $observer->getItem();


            $discount = $this->helperData->calculateDiscount($ruleSimpleAction,$ruleDiscount*$sameRuleCounter,$item,$observer);


            //register discount description
            $this->discountCollector->applyRuleAmount(str_replace(","," ",$ruleDescription),0);
            $observer->getResult()->setAmount($discount);
        }
    }

}
